/* Various defines: */
#define sign(x) (x > 0 ? 1 : -1)

#define abs(x) (x >= 0 ? x : -x)
#define min(a,b) (a<b ? a : b)

int bound(float x, float a, float b) {
	return (a<x && x<b);
}



template <class T>
class CircularBuffer
{
	int  n;
	T   *buf;
	int  last;

public:
	CircularBuffer(int _n) : n(_n), buf(new T[n]), last(0) {}
	~CircularBuffer() { delete[] buf; }
	T operator[](unsigned int i) const
		{ assert(i<n); int j=last-i; return buf[j<0 ? j+n : j]; }
	CircularBuffer<T> &operator <<(T x)
		{ last++; if (last==n) last = 0; buf[last]=x; return *this; }
	T leaving() { return (*this)[n-1]; }
};



class Exception
{
	char *reason;
public:
	Exception(char *r) { reason = strdup(r); }
//	virtual ~Exception() { fprintf(stderr, "axxxx\n"); free(reason); }
	void dump(ostream &o) const { o << reason; }
};


class InputException : public Exception
{
	int pos1, pos2;
public:
	InputException(char *r, int p) : Exception(r), pos1(p), pos2(p) {}
	InputException(char *r, int p1, int p2) : Exception(r), pos1(p1), pos2(p2) {}
	void dump(ostream &o) const
		{ Exception::dump(o);
		if (pos1==pos2) o<<" at "<<pos1;
		else o<<" in ["<<pos1<<","<<pos2<<"]"; }
};


class GiveUpInputEx : public InputException
{
public:
	GiveUpInputEx(char *r, int p) : InputException(r,p) {}
};


class GiveUpBlockEx : public InputException
{
public:
	GiveUpBlockEx(char *r, int p) : InputException(r,p) {}
	GiveUpBlockEx(char *r, int p1, int p2) : InputException(r,p1,p2) {}
};
/* End of defines */
