/* >>>>>>>>>>>>>>>>>>> The BBC Tape Utility <<<<<<<<<<<<<<<<<<<<<<<<<
 *
 * Copyright (c) Ole Stauning 1996.
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the copyright holder
 * not be used in advertising or publicity pertaining to distribution
 * of the software without specific, written prior permission. The
 * copyright holder makes no representations about the suitability of
 * this software for any purpose. It is provided "as is" without express
 * or implied warranty.
 *
 * THE COPYRIGHT HOLDER DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY
 * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
 * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
 * CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *  So if you fry your soundcard, do not blame it on me :-)
 *
 *  Please send comments, questions, bug-reports and flames to
 *
 *                        os@imm.dtu.dk
 */

#include <math.h>
#include <stdio.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <unistd.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <signal.h>

#include <sys/types.h>
#include <sys/wait.h>

#include <sys/soundcard.h>


#define DEFAULT_AUDIO_DEV "/dev/dsp"
#define DEFAULT_SAMPLE_FREQ 44100

int SAMPLE_FREQ = DEFAULT_SAMPLE_FREQ;
u_char *command;

#define true 1
#define false 0

#include "_bbcim_code.c"
//#include "_wav.c"
#include "_audio.c"
#include "_decode.c"
#include "_save.c"
#include "_help.c"


int getoptions(int argc, char *argv[]) {
	char c;
	int i;

	while ((c = getopt(argc, argv, "d:f:r:s:b:w:")) != EOF) {
		switch (c) {
		case 'd': /* change device name */
			strcpy(AUDIO_DEV,optarg);
			break;
		case 'f': /* change sample frequency */
			SAMPLE_FREQ = atoi(optarg);
			break;
		case 'r': /* change baud rate */
			BAUD_RATE = atoi(optarg);
			break;
		case 's': /* change block size */
			BLOCK_SIZE = atoi(optarg);
			break;
		case 'b': /* change start- and stop- bits */
			NUMBER_OF_STOP_BITS = 0;
			NUMBER_OF_START_BITS = 0;
			STOP_BITS = START_BITS = 0;
	
			for(i=0;optarg[i]!='/';i++)
				if (optarg[i]=='1')
					START_BITS |= 1<<i;
				else
					if (optarg[i]!='0') usage();

			if ((NUMBER_OF_START_BITS = i) == 0) {
				fprintf(stderr,"%s: you must have startbits\n",command);
				exit(-1);
			}
			for(i++;optarg[i]!='\0';i++)
				if (optarg[i]=='1')
					STOP_BITS |= 1<<(i-NUMBER_OF_START_BITS-1);
				else
					if (optarg[i]!='0') usage();

			if ((NUMBER_OF_STOP_BITS = i-NUMBER_OF_START_BITS-1) == 0) {
				fprintf(stderr,"%s: you must have stopbits\n",command);
				exit(-1);
			}

			if (START_BITS&1) {
				fprintf(stderr,"%s: first startbit has to be 0\n",command);
				exit(-1);
			}

			break;
		case 'w': /* load/save wav file */
			USE_WAV_FILE = 1;
			strcpy(WAV_FILE,optarg);
			break;
		default:
			usage();
		}
	}
	return optind;
}

void bbc_load(int argc, char *argv[]) {
	int optind, audio;

	optind = getoptions(argc,argv);
fprintf(stderr, "bbc_load (optind=%d)\n", optind);
	if (USE_WAV_FILE) {
		if (strstr(WAV_FILE,"-")) /* the wav file is coming from stdin */
			audio=0;
		else
			if ((audio = open(WAV_FILE, O_RDONLY, 0)) == -1) { /* the wav file is coming from disk */
				perror(WAV_FILE);
				exit(-1);
			}
//		test_wavefile(audio);
	}
	else {
		audio = audio_open(O_RDONLY); /* the wav file is coming from dsp */
	}

//	if (optind > argc - 1) { /* write data to stdout */
//		printf("writing to stdout\n");
//		data = 1;
//	}
//	else {
//		if ((data = open(argv[optind], O_WRONLY | O_CREAT, 0666)) == -1) { /* write data to file */
//			perror(argv[optind]);
//			exit(-1);
//		}
//		printf("writing to file %s\n", argv[optind]);
//	}
	load(audio);
//	if (data != 1) close(data);
//	if (audio != 0) close(audio);
}

void bbc_save(int argc, char *argv[]) {
	int idx, data, audio;

	idx = getoptions(argc,argv);

	if (idx > argc - 1) usage(); /* no tape filename specified */
	strcpy(TAPE_FILENAME, argv[idx++]);
	if (idx > argc - 1) usage(); /* no load address specified */
	LOAD_ADDRESS = strtol(argv[idx++], NULL, 0);
	if (idx > argc - 1) usage(); /* no exec address specified */
	EXEC_ADDRESS = strtol(argv[idx++], NULL, 0);

	if (USE_WAV_FILE) {
		if (strstr(WAV_FILE,"-")) /* the wav file sent to stdout */
			audio = 1;
		else
			if ((audio = open(WAV_FILE, O_WRONLY | O_CREAT, 0666)) == -1) { /* well, lets waste some HD :) */
				perror(WAV_FILE);
				exit(-1);
			}
//		write_wav_header(audio);
	}
	else audio = audio_open(O_WRONLY); /* the wav file sent to dsp */

	if (idx > argc - 1) /* read data from stdin */
		data = 0;
	else
		if ((data = open( argv[idx], O_RDONLY, 0)) == -1) { /* read data from file */
			perror(argv[idx]);
			exit(-1);
		}
	save(data,audio);
	if (data != 0) close(data);
	if (audio != 1) close(audio);
}

int main(int argc, char *argv[]) {

	command = argv[0];
	if (strstr (command, "bbcload")) bbc_load(argc, argv);
	else if (strstr (command, "bbcsave")) bbc_save(argc, argv);
	else {
		fprintf (stderr,"Error: command should be named either bbcload or bbcsave\n");
		exit (1);
	}
	return 1;
}

