/* _io.c
 *
 * By W.H.Scholten 2001
 * This file is released to the public domain.
 *
 * Read/write wrappers (signals etc. can cause short read/write counts)
 */

#include <sys/types.h>
#include <sys/uio.h>
#include <unistd.h>

#include <errno.h>

int my_write(int fd, void *data, int nbytes) {
	int bytes_written = 0;
	int bytes_left = nbytes;

	for (;;) {
		int st = write(fd, data, bytes_left);
		if (st < 0 && errno != EINTR) {
			//perror(write_file);
			//exit(1);
			return -1;
		}
		bytes_written += st;
		bytes_left -= st;
		if (bytes_written == nbytes) break;
	}

	return bytes_written;
}


#if 0
int my_read_byte(int fd) {
	unsigned char my_byte;
	for (;;) {
		int st = read(fd, &my_byte, 1);
		if (st < 0 && errno != EINTR) {
			perror("write byte");
			exit(1);
		}
		if (st == 1) break;
	}

	return my_byte;
}
#endif


int my_read(int fd, void *data, int nbytes) {
	int bytes_read = 0;
	int bytes_left = nbytes;

	for (;;) {
		int st = read(fd, data, bytes_left);
		if (st < 0 && errno != EINTR) {
			//perror(write_file);
			//exit(1);
			return -1;
		}
		bytes_read += st;
		bytes_left -= st;
		if (bytes_read == nbytes || st == 0) break; /* done or EOF */
	}

	return bytes_read;
}

int my_fwrite(FILE *fp, void *data, int length, char *fn) {
	int  st;
	st = fwrite(data, 1, length, fp);

	if (st != length) {
		perror(fn);
		exit(1);
	}

	return 0;
}


void bbctape_fatal_perror(char *s) {
	perror(s);
	exit(1);
}
